import{t as e}from"./index-CYR-HZan.js";import{p as n}from"./api-F8LEWdj1.js";function r({className:t,...o}){return e.jsx("kbd",{"data-slot":"kbd",className:n("bg-muted text-muted-foreground pointer-events-none inline-flex h-5 w-fit min-w-5 items-center justify-center gap-1 rounded-sm px-1 font-sans text-xs font-medium select-none","[&_svg:not([class*='size-'])]:size-3","[[data-slot=tooltip-content]_&]:bg-background/20 [[data-slot=tooltip-content]_&]:text-background dark:[[data-slot=tooltip-content]_&]:bg-background/10",t),...o})}export{r as K};
