#!/bin/bash
set -euo pipefail

apt-get update
apt-get install -y curl

curl https://cursor.com/install -fsS | bash

export PATH="$HOME/.local/bin:$PATH"
cursor-agent --version

echo "INSTALL SUCCESS!"
