window.__reactRouterManifest={"entry":{"module":"/assets/entry.client-Dk9HL2zZ.js","imports":["/assets/index-CYR-HZan.js"],"css":[]},"routes":{"root":{"id":"root","path":"","hasAction":false,"hasLoader":false,"hasClientAction":false,"hasClientLoader":false,"hasClientMiddleware":false,"hasErrorBoundary":true,"module":"/assets/root-B_7xc0ep.js","imports":["/assets/index-CYR-HZan.js","/assets/index-JVv6katd.js","/assets/mutation--h5hHClj.js","/assets/compare-Br3z3FUS-D09VrqaM.js"],"css":["/assets/root-BzmlCMuh.css"]},"routes/home":{"id":"routes/home","parentId":"root","index":true,"hasAction":false,"hasLoader":false,"hasClientAction":false,"hasClientLoader":false,"hasClientMiddleware":false,"hasErrorBoundary":false,"module":"/assets/home-QMMsjKRF.js","imports":["/assets/index-CYR-HZan.js","/assets/index-JVv6katd.js","/assets/api-F8LEWdj1.js","/assets/empty-DW6DWxQP.js","/assets/input-CJFbpsS_.js","/assets/hooks-CkUsEmdN.js","/assets/tooltip-C0rAc-Ow.js","/assets/kbd-BbT4rXhg.js","/assets/mutation--h5hHClj.js","/assets/index-Bn6L4jjc.js","/assets/compare-Br3z3FUS-D09VrqaM.js","/assets/scroll-area-6vQWqo-y.js"],"css":[]},"routes/compare":{"id":"routes/compare","parentId":"root","path":"compare","hasAction":false,"hasLoader":false,"hasClientAction":false,"hasClientLoader":false,"hasClientMiddleware":false,"hasErrorBoundary":false,"module":"/assets/compare-C_0KNNt_.js","imports":["/assets/index-CYR-HZan.js","/assets/api-F8LEWdj1.js","/assets/index-JVv6katd.js","/assets/index-Bn6L4jjc.js","/assets/kbd-BbT4rXhg.js","/assets/tooltip-C0rAc-Ow.js"],"css":[]},"routes/job":{"id":"routes/job","parentId":"root","path":"jobs/:jobName","hasAction":false,"hasLoader":false,"hasClientAction":false,"hasClientLoader":false,"hasClientMiddleware":false,"hasErrorBoundary":false,"module":"/assets/job-D-y46ccV.js","imports":["/assets/index-CYR-HZan.js","/assets/api-F8LEWdj1.js","/assets/index-JVv6katd.js","/assets/empty-DW6DWxQP.js","/assets/input-CJFbpsS_.js","/assets/tooltip-C0rAc-Ow.js","/assets/code-block-PcQLF1iv.js","/assets/hooks-CkUsEmdN.js","/assets/tabs-CzM9Sz0e.js","/assets/kbd-BbT4rXhg.js","/assets/mutation--h5hHClj.js","/assets/index-Bn6L4jjc.js","/assets/compare-Br3z3FUS-D09VrqaM.js","/assets/scroll-area-6vQWqo-y.js"],"css":[]},"routes/task":{"id":"routes/task","parentId":"root","path":"jobs/:jobName/tasks/:source/:agent/:modelProvider/:modelName/:taskName","hasAction":false,"hasLoader":false,"hasClientAction":false,"hasClientLoader":false,"hasClientMiddleware":false,"hasErrorBoundary":false,"module":"/assets/task-pckVMxYH.js","imports":["/assets/index-CYR-HZan.js","/assets/api-F8LEWdj1.js","/assets/code-block-PcQLF1iv.js","/assets/hooks-CkUsEmdN.js","/assets/kbd-BbT4rXhg.js","/assets/index-JVv6katd.js","/assets/scroll-area-6vQWqo-y.js"],"css":[]},"routes/trial":{"id":"routes/trial","parentId":"root","path":"jobs/:jobName/tasks/:source/:agent/:modelProvider/:modelName/:taskName/trials/:trialName","hasAction":false,"hasLoader":false,"hasClientAction":false,"hasClientLoader":false,"hasClientMiddleware":false,"hasErrorBoundary":false,"module":"/assets/trial-Ml11EUC_.js","imports":["/assets/index-CYR-HZan.js","/assets/api-F8LEWdj1.js","/assets/index-JVv6katd.js","/assets/empty-DW6DWxQP.js","/assets/tabs-CzM9Sz0e.js","/assets/index-Bn6L4jjc.js","/assets/scroll-area-6vQWqo-y.js","/assets/code-block-PcQLF1iv.js","/assets/mutation--h5hHClj.js"],"css":[]}},"url":"/assets/manifest-2885d6a2.js","version":"2885d6a2"};