import os

from dotenv import load_dotenv

load_dotenv()

HARBOR_SUPABASE_URL = os.environ.get(
    "HARBOR_SUPABASE_URL", "https://hlqxxzsirfrgeqasvaps.supabase.co"
)
HARBOR_SUPABASE_PUBLISHABLE_KEY = os.environ.get(
    "HARBOR_SUPABASE_PUBLISHABLE_KEY", "sb_publishable_RGMKhIM1NKzZ8bL3qCVNuA_5TsD6rbF"
)
